using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace ScaleTextToViewport
{
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        const float SPEED = 0.5f / 1000;      // laps per millisecond
        const string TEXT = "Hello, Windows Phone!";

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        SpriteFont kootenay14;
        Vector2 textPosition;
        Vector2 origin;
        Vector2 maxScale;
        Vector2 scale;
        float angle;
        float tLap;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.IsFullScreen = true;       // TODO: Remove when ClientBounds is working!
            Content.RootDirectory = "Content";
            TargetElapsedTime = TimeSpan.FromTicks(333333);

            graphics.PreferredBackBufferWidth = 480;
            graphics.PreferredBackBufferHeight = 800;
        }

        protected override void Initialize()
        {
            base.Initialize();
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);
            Rectangle clientBounds = this.Window.ClientBounds;

            kootenay14 = this.Content.Load<SpriteFont>("Kootenay14");
            Vector2 textSize = kootenay14.MeasureString(TEXT);
            textPosition = new Vector2(clientBounds.Left + clientBounds.Width / 2, 
                                       clientBounds.Top + clientBounds.Height / 2);
            origin = new Vector2(textSize.X / 2, textSize.Y / 2);

            // flip 90 degrees if portrait mode
            if (clientBounds.Width < clientBounds.Height)
            {
                angle = MathHelper.PiOver2;
                maxScale = new Vector2(clientBounds.Height / textSize.X,
                                       clientBounds.Width / textSize.Y);
            }
            else
            {
                maxScale = new Vector2(clientBounds.Width / textSize.X,
                                       clientBounds.Height / textSize.Y);
            }
        }

        protected override void UnloadContent()
        {
        }

        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            tLap = (SPEED * (float)gameTime.TotalGameTime.TotalMilliseconds) % 1;
            float pLap = (1 - (float)Math.Cos(tLap * MathHelper.TwoPi)) / 2;
            scale = Vector2.Lerp(Vector2.One, maxScale, pLap);

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Navy);

            spriteBatch.Begin();
            spriteBatch.DrawString(kootenay14, TEXT, textPosition, Color.White,
                                   angle, origin, scale, SpriteEffects.None, 0);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
