using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace VectorTextMovement
{
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        const float SPEED = 240f / 1000;      // pixels per millisecond
        const string TEXT = "Hello, Windows Phone!";

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        SpriteFont kootenay14;
        Vector2 midPoint;
        Vector2 pathVector;
        Vector2 pathDirection;
        Vector2 textPosition;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.IsFullScreen = true;       // TODO: Remove when ClientBounds is working!
            Content.RootDirectory = "Content";
            TargetElapsedTime = TimeSpan.FromTicks(333333);

            graphics.PreferredBackBufferWidth = 480;
            graphics.PreferredBackBufferHeight = 800;
        }

        protected override void Initialize()
        {
            base.Initialize();
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);
            Rectangle clientBounds = this.Window.ClientBounds;

            kootenay14 = this.Content.Load<SpriteFont>("Kootenay14");
            Vector2 textSize = kootenay14.MeasureString(TEXT);

            Vector2 position1 = new Vector2(clientBounds.Right - textSize.X, clientBounds.Top);
            Vector2 position2 = new Vector2(clientBounds.Left, clientBounds.Bottom - textSize.Y);
            midPoint = Vector2.Lerp(position1, position2, 0.5f);

            pathVector = position2 - position1;
            pathDirection = pathVector;
            pathDirection.Normalize();
            textPosition = position1;
        }

        protected override void UnloadContent()
        {
        }

        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            float pixelChange = SPEED * (float)gameTime.ElapsedGameTime.TotalMilliseconds;
            textPosition += pixelChange * pathDirection;

            if ((textPosition - midPoint).LengthSquared() >
                                                (0.5f * pathVector).LengthSquared())
            {
                float excess = (textPosition - midPoint).Length() -
                                                (0.5f * pathVector).Length();
                pathDirection = -pathDirection;
                textPosition += 2 * excess * pathDirection;
            }

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Navy);

            spriteBatch.Begin();
            spriteBatch.DrawString(kootenay14, TEXT, textPosition, Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}

