using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace CarOnPolylineCourse
{
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        const float SPEED = 0.25f / 1000; // laps per millisecond
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Texture2D car;
        Vector2 carCenter;
        PolylineInterpolator polylineInterpolator = new PolylineInterpolator();
        Vector2 position;
        float rotation;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.IsFullScreen = true;       // TODO: Remove when ClientBounds is working!
            Content.RootDirectory = "Content";
            TargetElapsedTime = TimeSpan.FromTicks(333333);

            graphics.PreferredBackBufferWidth = 480;
            graphics.PreferredBackBufferHeight = 800;
        }

        protected override void Initialize()
        {
            base.Initialize();
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);
            car = this.Content.Load<Texture2D>("Car");
            carCenter = new Vector2(car.Width / 2, car.Height / 2);
            float margin = car.Width;
            Rectangle clientBounds = this.Window.ClientBounds;

            polylineInterpolator.Vertices.Add(
                new Vector2(clientBounds.Left + car.Width, clientBounds.Top + car.Width));
            polylineInterpolator.Vertices.Add(
                new Vector2(clientBounds.Right - car.Width, clientBounds.Top + car.Width));
            polylineInterpolator.Vertices.Add(
                new Vector2(clientBounds.Left + car.Width, clientBounds.Bottom - car.Width));
            polylineInterpolator.Vertices.Add(
                new Vector2(clientBounds.Right - car.Width, clientBounds.Bottom - car.Width));
            polylineInterpolator.Vertices.Add(
                new Vector2(clientBounds.Left + car.Width, clientBounds.Top + car.Width));
        }

        protected override void UnloadContent()
        {
        }

        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            float t = (SPEED * (float)gameTime.TotalGameTime.TotalMilliseconds) % 1;
            float angle;
            position = polylineInterpolator.GetValue(t, false, out angle);
            rotation = angle + MathHelper.PiOver2;

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Blue);

            spriteBatch.Begin();
            spriteBatch.Draw(car, position, null, Color.White, rotation,
                             carCenter, 1, SpriteEffects.None, 0);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
